'***********************************************************************
'      PRIMJER PROGRAMA ZA AUTOKALIBRACIJU VIE IZABRANIH ADC ULAZA

'Mikrokontroler: ATtiny84

'Koristi knjizicu "Multi Adcx.inc".

'Za svaki ulaz Adcn (n = 0 do max.7) kalibracija se provodi

'u dvije toke (Minimum i Maximum).

'Nakon kalibracije i ADC pretvorbe, te naredbe Adc_correction Adcn slijedi:

'Naponski signal < Minimum --> Adcx(adcn) = 0 ;

'Under_min(adcn) = 1 ; Over_max(adcn) = 0

'Naponski signal > Maximum --> Adcx(adcn)=1023 ;

'Under_min(adcn) = 0 ; Over_max(adcn) = 1

'Naponski signal >= Minimum i <= Maximum --> Adcx(adcn) = 0 ... 1023 ;

'Under_min(adcn) = 0 i Over_max(adcn) = 0

'Varijable Adcx(adcn) su Single varijable ; Adcx(adc0) ... Adcx(adc7)
'--------------------------------------------------------------------
'LCD je spojen paralelno u 4 bitnom nainu rada.

'U ovom primjeru pomou 2 potenciometara koji su spojeni na ulaze
'ADC0 i ADC1 simuliraju se mjerni pretvornici temperature.

'Toke kalibracije za ulaz ADC0:
'1.toka kalibracije (Minimum):0C,napon za temperaturu 0C je 1V
'2.toka kalibracije (Maximum):100C,napon za temperaturu 100C je 3V

'Toke kalibracije za ulaz ADC1:
'1.toka kalibracije (Minimum):-20C,napon za temperaturu -20C je 0,5V
'2.toka kalibracije (Maximum):50C,napon za temperaturu 50C je.4,5V

'Autor: Mladen empuh , Datum 12.07.2023. e-mail: cempuh.mladen@gmail.com
'*************************************************************************
$regfile = "attiny84.dat"                                   ' ATtiny84

$crystal = 8000000                                          ' 8MHz
'$crystal = 1000000                                          ' 1MHz

$swstack = 50                                               ' SW stack=50 bytes,po potrebi poveaj
$hwstack = 40                                               ' HW stack=40 bytes,po potrebi poveaj
$framesize = 64

'------------------------------------------------------------
'        DEFINIRANJE PINOVA ZA LCD U 4 BITNOM RADU             Korigiraj prema svom rasporedu pinova za LCD !!!
'------------------------------------------------------------
Config Lcd = 16 * 2
Config Lcdpin = Pin , Db4 = Porta.5 , Db5 = Porta.6 , Db6 = Porta.7 , Db7 = Portb.2 , Rs = Porta.4 , E = Porta.3
Cls
Cursor Off

'------------------------------------------------------------
'        INICIJALIZACIJA AD KONVERTERA
'------------------------------------------------------------
Config Adc = Single , Prescaler = Auto , Reference = Avcc   'inicijaliziraj ADC za pretvorbu od 0 do Avcc=5V

Start Adc                                                   ' startaj ADC pretvorbu

'------------------------------------------------------------

Channel_enable Alias &B00000011                             ' omogueni ADC ulazi ADC0 i ADC1 za kalibraciju

'------------------------------------------------------------
'Izaberi ulaz za kalibracijsku tipku:

Calibration_key Alias Pinb.0                                ' kalibracijska tipka Calibration_key spojena je na ulaz PB.0

Config Calibration_key = Input                              ' naziv kalibracijske tipke mora biti Calibration_key

Portb.0 = 1                                                 ' ukljuenje pull up otpornika na ulazu za kalibracijsku tipku

'-----------------------------------------------------------
'Izaberi ulaz za tipku za selektiranje kanala za kalibraciju:

Select_key Alias Pinb.1                                     ' selekcijska tipka Select_key spojena je na ulaz PB.1

Config Select_key = Input                                   ' naziv selekcijske tipke mora biti Select_key

Portb.1 = 1                                                 ' ukljuenje pull up otpornika na ulazu za selekcijsku tipku

'-----------------------------------------------------------

$include "Multi Adcx.inc"                                   ' ukljui knjiicu "Multi Adcx.inc" za ADC kalibraciju izabranih ADC ulaza
                                                             ' i naredbu Adc_correction Adcn ; n = 0 do 7
'-----------------------------------------------------------

Dim Temp1 As Single , Temp2 As Single                       ' Single varijable za izraun temperatura

Do

  Adc_input(adc0) = Getadc(0)                               ' uitaj analogni ulaz ADC0, naziv varijable za uitanje tog ulaza mora biti Adc_input(adc0)

  Adc_correction Adc0                                       ' korekcija iznosa Adc_input(adc0) sukladno kalibraciji i upis u Single varijablu Adcx(adc0)

  If Under_min(adc0) = 0 And Over_max(adc0) = 0 Then        ' ako je ulazni napon unutar mjernog opsega kalibracije tada
                                                            ' izraunaj i ispii temperaturu Temp1
   Temp1 = Adcx(adc0) * 100
   Temp1 = Temp1 / 1023                                     ' Temp1 = Adcx(adc0)*(100-0)/1023 + 0 C  ; M.O.K. je od 0C do 100C

   Locate 1 , 1 : Lcd "Temp1 = " ; Fusing(temp1 , "#.#") ; Chr(223) ; "C  "
  End If

  If Under_min(adc0) = 1 Then                               ' ulazni napon na ADC0 ulazu je ispod mjernog opsega kalibracije
   Locate 1 , 1 : Lcd "Temp1 < 0" ; Chr(223) ; "C     "     ' ispii da je Temp1 < 0 C
  End If

  If Over_max(adc0) = 1 Then                                ' ulazni napon na ADC0 ulazu je iznad mjernog opsega kalibracije
   Locate 1 , 1 : Lcd "Temp1 > 100" ; Chr(223) ; "C   "     ' ispii da je Temp1 > 100 C
  End If

  Adc_input(adc1) = Getadc(1)                               'uitaj analogni ulaz ADC1, naziv varijable za uitanje tog ulaza mora biti Adc_input(adc1)

  Adc_correction Adc1                                       ' korekcija iznosa ulaza Adc_input(adc1) sukladno kalibraciji i upis u Single varijablu Adcx(adc1)

  If Under_min(adc1) = 0 And Over_max(adc1) = 0 Then        ' ako je ulazni napon na ADC1 ulazu  unutar mjernog opsega kalibracije tada
                                                              ' izraunaj i ispii temperaturu Temp2
   Temp2 = Adcx(adc1) * 70
   Temp2 = Temp2 / 1023
   Temp2 = Temp2 - 20                                       ' Temp2 = Adcx(adc1)*(50-(-20))/1023 + (-20) C ; M.O.K. je od -20C do 50C

   Locate 2 , 1 : Lcd "Temp2 = " ; Fusing(temp2 , "#.#") ; Chr(223) ; "C   "
  End If

  If Under_min(adc1) = 1 Then                               ' ulazni napon na ADC1 ulazu je ispod mjernog opsega kalibracije
   Locate 2 , 1 : Lcd "Temp2 < -20" ; Chr(223) ; "C   "     ' ispii da je Temp2 < -20 C
  End If

  If Over_max(adc1) = 1 Then                                ' ulazni napon na ADC1 ulazu je iznad mjernog opsega kalibracije
   Locate 2 , 1 : Lcd "Temp2 > 50" ; Chr(223) ; "C    "     ' ispii da je Temp2 > 50 C
  End If

  Waitms 500                                                ' opciona pauza od 500ms za usporavanje uestalosti ispisa temperatura u Do-Loop petlji

Loop

'------------------------------------------------------------
End                                                         ' kraj programa