'**************************************************************************************
'                       AUTOKALIBRACIJA IZABRANIH ADC ULAZA

'Naziv: Multi Adcx.inc

'Kalibracija se provodi u dvije tocke (minimum i maksimum signala).

'Podrzava kalibraciju do maksimum 6 ili 8 izabranih ADC ulaza (ovisno o chipu).

'ADC ulazi koji se kalibriraju izabrani su sa vrijednosti iznosa

'Channel_enable Alias &Bxxxxxxxx. Svaki bit koji u toj vrijednosti ima stanje 1

'omogucava pripadajucem ADC ulazu kalibraciju (bit 0 za ADC0 do bita 7 za ADC7).

'Ako je bit u stanju 0 pripadajuci ADC ulaz se ne kalibrira.

'Ulaz ADC0, ako je izabran za kalibraciju, treba naredbom Getadc(0) ucitati u

'varijablu Adc_input(adc0) i tako opciono sve do ADC7 kojeg se naredbom Getadc(7),

'ako postoji ovisno o chipu i ako je omogucen za kalibraciju, ucitava u Adc_input(adc7).

'Tipka na ulazu Select_key je za izbor ADC ulaza koji se kalibrira (0 do max.7)

'Tipka na ulazu Calibration_key je za ulaz u kalibraciju i za prihvat podataka

'kalibracije i za izlaz iz kalibracije, nakon zavrsetka kalibracije izabranog ulaza.

'Kalibracijski podaci Offset(adcn) i Gain(adcn) za svaki kalibrirani ulaz

'spremaju se u interni EEPROM, n je redni broj ADC ulaza od 0 do max.7

'Naredba Adc_correction Adcn, pri cemu je n redni broj ADC ulaza (0 do max.7),

'je za korekciju iznosa Adc_input(adcn) sukladno kalibracijskim parametrima i

'rezultat korekcije upisuje u Single varijablu Adcx(adcn),n je redni broj

'ADC ulaza od 0 do  max.7

'Offset(adcn) je Word varijabla, pa Offset(adcn) moze imati iznos i veci od 255.

'Za Adc_input(adcn) < Offset(adcn) je Adcx(adcn)=0;Under_min(adcn)=1;Over_max(adcn)=0.

'Za Adc_input(adcn) > kalibracijskog maksimuma za taj ADC ulaz je Adcx(adcn)=1023 ;

'Under_min(adcn)=0 ; Over_max(adcn)=1.

'Za Adc_input(adcn) >= Offset(adcn) i <= kalibracijkom maksimumu za taj ADC ulaz

'je Adcx(adcn)=0...1023 (Single varijabla); Over_max(adcn)=0 ; Under_min(adcn)=0

'Autor: Mladen Cempuh , Datum 21.10.2022. e-mail: cempuh.mladen@gmail.com
'**************************************************************************************

Config Submode = New
'------------------------------------------------------------

#if _chip = 17 Or _chip = 60 Or _chip = 61 Or _chip = 34    'ATmega8/8A ; ATmega328/328P ; ATmega168P ; ATmega168
Channel_max Alias 6                                         ' Max. 6 ADC kanala: ADC0 do ADC5
#else
Channel_max Alias 8                                         ' Max. 8 ADC kanala: ADC0 do ADC7
#endif

'-------------------------------------------------------------
Dim Adc_input(channel_max) As Word                          ' Niz Word varijabli za podatke ADC sa izabranih ulaza
Dim Adcx(channel_max) As Single                             ' Niz Single varijabli za podatke nakon korekcije ADC podataka
Dim Offset(channel_max) As Word                             ' Niz Word varijabli za offsete tj. ADC iznose kod minimalnih ulaznih napona
Dim Gain(channel_max) As Single                             ' Niz Single varijabli za korekcije pojacanja (nagib pravca) izabranih ulaza
Dim Over_max(channel_max) As Byte                           ' Niz Byte varijabli za Over_max
Dim Under_min(channel_max) As Byte                          ' Niz Byte varijabli za Under_min

Dim Calibrated_ok As Byte                                   ' Varijabla za potvrdu izvrsenja kalibracije svih izabranih ulaza, 1=izvrseno
Dim Ch_enable As Byte                                       ' Varijabla za ucitanje definirane vrijednosti Channel_enable
Dim Channel As Byte                                         ' Byte varijabla za redni broj izabranog kanala
Dim Adcn_calibrated As Byte                                 ' kalibrirani ADC ulazi u toj varijabli imaju pripadajuci bit 0
Dim Adcn As Byte                                            ' Varijabla za trenutno izabrani ADC ulaz
Dim Next_adc_read As Word                                   ' Varijabla za N uzastopnih citanja ADC ulaza Channel za odredjivanje maksimuma Offseta
Dim Temp_word As Word                                       ' Pomocna Word varijabla

'EEPROM varijable internog EEPROM-a:

Dim Not_used As Eram Byte                                   ' interna EEPROM lokacija na adresi 0 ne smije se koristi (nepouzdana)
Dim Calibrated_ok_eeprom As Eram Byte                       ' ako je 1 kalibracija izvrsena
Dim Offset_eeprom(channel_max) As Eram Word                 ' Niz Word varijabli EEPROM-a za pohranu Offset iznosa
Dim Gain_eeprom(channel_max) As Eram Single                 ' Niz Single varijabli EEPROM-a za pohranu Gain iznosa
Dim Adcn_calibrated_eeprom As Eram Byte                     ' Varijabla EEPROM-a za spremanje stanja varijable Adcn_calibrated

'-------------------------------------------------------------
Adc0 Alias 1 : Adc1 Alias 2 : Adc2 Alias 3 : Adc3 Alias 4
Adc4 Alias 5 : Adc5 Alias 6 : Adc6 Alias 7 : Adc7 Alias 8

Ch_enable = Channel_enable                                  ' u varijablu Ch_enable ucitaj iznos Channel_enable

If Ch_enable = 0 Then                                       ' ako je 0 tada
 Cls : Lcd "Channel_enable"                                 ' netocno upisan iznos u Channel_enable,
 Locate 2 , 1 : Lcd "incorrect value!"                      ' jer nije omogucen niti jedan ADC ulaz za kalibraciju

Lab_wait_for_reset:
Goto Lab_wait_for_reset                                     ' cekaj Reset

End If

Calibrated_ok = Calibrated_ok_eeprom                        ' ucitaj u varijablu Calibrated_ok iznos iz EEPROM varijable Calibrated_ok_eeprom

If Calibrated_ok = 255 Then                                 ' EEPROM obrisan , treba kalibrirati

 For Channel = 1 To Channel_max

  Offset_eeprom(channel) = 0                                ' upisi startno 0 u sve Offset i Gain EEPROM varijable

  Gain_eeprom(channel) = 0

 Next Channel

 Adcn_calibrated = Ch_enable                                ' Adcn_calibrated se puni iznosom Ch_enable

 Gosub Calibration                                          ' idi na prvu kalibraciju omogucenih ADC ulaza

 Goto Lab_load_data                                         ' idi na Lab_load_data

End If

Cls : Lcd "Calibration key"                                 ' ispisi "Calibration key 5s for calibrate"

Locate 2 , 1 : Lcd "5s for calibrate"

Wait 2                                                      ' pauza 2s

Lab_4_5s_wait:

   If Calibration_key = 1 Then Goto Lab_load_data           ' i nije pritisnuta tipka Calibration_key idi na Lab_load_data

   Incr Temp_word

   Waitms 100

   If Temp_word < 30 Then Goto Lab_4_5s_wait                'dodatno cekaj jos 3s pritisnutu tipku Calibration_key za ulazak u kalibraciju

If Calibrated_ok = 1 Then

   Adcn_calibrated = 0                                      '  ako je Calibrated_ok=1 tada je Adcn_calibrated je 0 , svi ADC ulazi kalibrirani-> rekalibracija

Else

  Adcn_calibrated = Adcn_calibrated_eeprom                  ' u protivnom se Adcn_calibrated se puni iz EEPROM varijable Adcn_calibrated_eeprom za nastavak kalibracije

End If

   Gosub Calibration                                        ' tipka Calibration_key je  bila pritisnuta 5s, idi na nastavak kalibracije ili rekalibraciju omogucenih ADC ulaza

Lab_load_data:

For Channel = 1 To Channel_max

   Offset(channel) = Offset_eeprom(channel)                 ' ucitaj sve Offset iznose iz EEPROM-a

   Gain(channel) = Gain_eeprom(channel)                     ' ucitaj sve Gain iznose iz EEPROM-a

Next Channel

Cls
                                                             ' obrisi LCD
Goto Lab_exit_calibration                                   ' izlaz iz datoteke "Multi Adcx.inc"

Sub Adc_correction(byval N As Byte)                         ' naredba Adc_correction n upisuje rezultat korekcije ADC kanala Adc_input(n) u Single varijablu Adcx(n); n=1 do max.8
 Over_max(n) = 0 : Under_min(n) = 0                         ' pocetno postavi Byte varijable Over_max(n) i Under_min(n) u 0
 If Adc_input(n) < Offset(n) Then                           ' za Adc_input(n) < Offset(n) je Byte varijabla Under_min(n)=1 i Single varijabla Adcx(n) = 0
    Under_min(n) = 1
    Adcx(n) = 0
    Goto Lab_exit_sub
 End If
 Temp_word = Adc_input(n) - Offset(n)
 Adcx(n) = Gain(n) * Temp_word                              ' Adcx(n) = Gain(n) * (Adc_input(n)-Offset(n))
 If Adcx(n) > 1023 Then                                     ' ako je Adcx(n) > 1023 (tada je Adc_input(n) > kalibracijskog maksimuma n-tog kanala)
  Over_max(n) = 1                                           ' tada je Byte varijabla Over_max(n)=1
  Adcx(n) = 1023                                            ' i Single varijabla Adcx(n) je 1023
 End If
Lab_exit_sub:
End Sub

'------------------------------------------------------------
'Potprogram za kalibraciju izabranog ADC ulaza

Calibration:

Cls : Lcd ">>Calibration<<" : Wait 3                        ' ispisi na LCD pocetak ciklusa kalibracije
Gosub Calibration_key_free                                  ' cekaj otpustanje kalibracijske tipke
Cls
Adcn = 0                                                    ' startno izabran ulaz ADC0
Channel = 1                                                 ' to je ADC kanal 1

Lab_select_adc:
Locate 1 , 1 : Lcd "Select: ADC" ; Adcn                     ' ispis trenutno izabranog ADC ulaza
Locate 2 , 1 : Lcd "Calib.key = YES"                        ' i da se za prihvat selektiranog ulaza potvrdjuje sa Calibration_key

Debounce Select_key , 0 , Next_adc , Sub                    ' sa Select_key idi na potprogram Next_adc za kruzno inkrementiranje ADC ulaza i kanala

If Calibration_key = 1 Then Goto Lab_select_adc             ' kalibracijska tipka nije pritisnuta, idi na Lab_select_adc

Gosub Calibration_key_free                                  ' kalibracijska tipka pritisnuta, cekaj otpustanje kalibracijske tipke

If Ch_enable.adcn = 0 Then                                  'izabrani ADC ulaz nije omogucen za kalibraciju
 Cls : Lcd "ADC" ; Adcn ; " not enabled"
 Locate 2 , 1 : Lcd "for calibration!"
 Wait 3                                                     ' pauza 3s
 Cls
 Goto Lab_select_adc                                        ' idi na ponovnu izbor ADC ulaza
End If

Cls : Lcd "ADC" ; Adcn ; " selected"                        ' u protivnom izabran je ADCn ulaz

Locate 2 , 1:
If Adcn_calibrated.adcn = 0 Then                            ' vec je prethodno bio kalibriran
 Lcd "Calibrate again?"                                     ' pa postavi pitanje  da li se eli taj ulaz ponovno kalibrirati
Else
 Lcd "for calibration"                                      ' ili je nekalibriran pa je za kalibraciju
End If

Wait 3                                                      ' pauza 3s

Gosub Select_key_free                                       ' ako je mozda pritisnuta tipka Select_key cekaj otpustanje tipke, brie se LCD

Lcd "Calib.key = YES"                                       ' izbornik: sa Calibration_key  prihvati startanje kalibracije izabranog ADCn ulaza
Locate 2 , 1 : Lcd "Select key = NO"                        '           sa Select_key ne prihvacanje startanja  kalibraciju izabranog ADCn ulaza

Lab1_wait_for_press_keys:

If Calibration_key = 0 Then Goto Lab_adcn_calibration       ' ako je pritisnuta tipka Calibration_key idi na Lab_adcn_calibration za start kalibracije

If Select_key = 0 Then                                      ' ako je pritisnuta tipka Select_key tada
 Gosub Select_key_free                                      ' cekaj otpustanje tipke Select_key i brie se LCD
 Lcd "Select = NEXT"                                        ' izbornik: "Select = NEXT"
 If Calibrated_ok <> 255 Then                               ' ako je Calibrated_ok<>255, kalibriran je barem jedan od omogucenih ADC ulaza
  Locate 2 , 1 : Lcd "Calib.key = EXIT"                     ' tada se izborniku dodaje opcija "Calib.key = EXIT" za moguci izlaz iz kalibracije
 End If

Lab_wait_for_press_key:

 If Calibrated_ok <> 255 Then                               ' ako je Calibrated_ok<>255, kalibriran je barem jedan od omogucenih ADC ulaza tada
  If Calibration_key = 0 Then Return                        ' ako se pritisne tipka Calibration_key izlazi se iz kalibracije bez izvrsenja kalibracije
 End If                                                     ' izabranog ADC ulaza i njegov kalibracijski status ostaje nepromijenjen
 If Select_key = 0 Then                                     ' ako se pritisne tipka Select_key tada
  Gosub Select_key_free                                     ' cekaj otpustanje tipke Select_key i brise se LCD
  Goto Lab_select_adc                                       ' i idi na Lab_select_adc za ponovnu selekciju ADC ulaza
 End If
 Goto Lab_wait_for_press_key                                ' idi na Lab_wait_for_press_key, ceka se pritisak jedne od te dvije tipke
End If

Goto Lab1_wait_for_press_keys                               ' idi na Lab1_wait_for_press_keys i cekaj pritisak jedne od te dvije tipke

Lab_adcn_calibration:                                       ' kalibracija izabranog Adcn ulaza

Gosub Calibration_key_free                                  ' cekaj otpustanje kalibracijske tipke

Cls : Lcd "ADC" ; Adcn ; " calibration"                     ' ispisi Adcn ulaz koji se kalibrira

Wait 3                                                      ' pauza 3s

Cls : Locate 2 , 1 : Lcd "Calib.key for OK"                 ' obrisi LCD i u drugom redu ispisi "Calib.key for OK"

'Prikljuci na izabrani ADC ulaz iznos mjernog signala koji odgovara minimalnom iznosu mjerene velicine:1.tocka kalibracije=Offset

Lab_first_point:
Temp_word = 0                                               ' postavi pomocnu varijablu Temp_word u 0
For Next_adc_read = 1 To 400                                ' citaj 400 puta izabran ADCn ulaz i pronadji maksimum za Offset(channel)
 Adc_input(channel) = Getadc(adcn)                          ' ucitaj sa ulaza ADCn u varijablu Adc_input(channel)
 If Adc_input(channel) > Temp_word Then Temp_word = Adc_input(channel)       ' nadji maksimalnu vrijednost signala na ADC ulazu i spremi ga u Temp_word
Next Next_adc_read                                          ' slijedece citanje ADCn ulaza

Locate 1 , 1 : Lcd "Minimum: " ; Temp_word ; "   "          'ispisi u prvom redu najveci iznos za Minimum iz varijable Temp_word
If Calibration_key = 0 Then                                 ' potvrdi pritiskom na kalibracijsku tipku
  Offset(channel) = Temp_word                               ' to je Offset iznos za ADCn ulaz
  Gosub Calibration_key_free                                ' cekaj otpustanje kalibracijske tipke
  Goto Lab_second_point                                     ' i idi na drugu tocku kalibracije
End If
Goto Lab_first_point

'Prikljuci na izabrani ADC ulaz iznos mjernog signala koji odgovara maksimalnom iznos mjerene velicine: 2. tocka kalibracije za Gain izracun

Lab_second_point:
Adc_input(channel) = Getadc(adcn)                           ' ucitaj sa ulaza ADCn u varijablu Adc_input(channel)
Locate 1 , 1 : Lcd "Maximum: " ; Adc_input(channel) ; "   " 'ispisi ADC iznos koji odgovara maksimumu
                                                             ' kod kojeg zelis da nakon korekcije  naredbom Adc_correction(adcn) iznos ADC pretvorbe bude 1023
If Calibration_key = 0 And Adc_input(channel) > Offset(channel) Then       ' ako je na izabrani ADCn ulaz spojen naponski signal kome je ADC iznos  Offseta
  Temp_word = Adc_input(channel) - Offset(channel)          ' i ako je pritisnuta kalibracijska tipka, tada izracunaj iznos za pripadajuci  Gain(channel)
  Gain(channel) = 1023 / Temp_word                          ' Gain(channel) = 1023 / (Adc_input(channel) - Offset(channel))
  Gosub Calibration_key_free                                ' cekaj otpustanje kalibracijske tipke
  Goto Save_calibration                                     ' idi na upis Offset(channel) i Gain(channel) iznosa u interni EEPROM
End If
Goto Lab_second_point                                       'tipka Calibration_key nije bila pritisnuta i nije se prihvatio maksimum,idi na Lab_second_point

Save_calibration:

 Offset_eeprom(channel) = Offset(channel)                   ' upis Offset(channel) iznosa u EEPROM
 Gain_eeprom(channel) = Gain(channel)                       ' upis Gain(channel) iznosa u EEPROM
 Reset Adcn_calibrated.adcn                                 ' resetiraj pripadajuci bit za kalibriran ADCn ulaz u varijabli Adcn_calibrated

Cls : Lcd "ADC" ; Adcn ; " calibrated!" : Wait 3            ' ispisi 3s da je ADCn ulaz kalibriran
Cls : Lcd "Select = NEXT"                                   ' izbornik: "Select = NEXT"
Locate 2 , 1 : Lcd "Calib.key = EXIT"                       '           "Calib.key = EXIT"
                                                             ' pritiskom na Select_key nastavlja se selekcija nekog ADC ulaza za kalibraciju
       '                                                     ' a pritiskom na Calibration_key  se izlazi iz kalibracije uz spremanje kalibracijskih
                                                            ' podataka za taj ADC ulaz i kalibracijskog statusa toga ulaza u EEPROM

Lab2_wait_for_press_keys:

If Calibration_key = 0 Then Goto Lab_calibration_end        ' sa Calibration_key idi na zavrsetak kalibracije

If Select_key = 0 Then                                      ' ako je pritisnuta tipka Select_key tada
 Gosub Select_key_free                                      ' cekaj otpustanje tipke Select_key
 Goto Lab_select_adc                                        ' idi na Lab_select_adc za ponovnu selekciju nekog ADC ulaza za kalibraciju
End If

Goto Lab2_wait_for_press_keys                               ' tipke nisu pritisnute ; idi na Lab2_wait_for_press_keys

Lab_calibration_end:                                        ' zavrsetak kalibracije izabranog ADC ulaza
If Adcn_calibrated = 0 Then                                 ' ako su sad svi omoguceni ADC ulazi kalibrirani tada
  Calibrated_ok = 1                                         ' je Calibrated_ok=1
 Else
  Calibrated_ok = 2                                         ' ako nisu svi omoguceni ADC ulazi kalibrirani tada je Calibrated_ok=2
End If

Adcn_calibrated_eeprom = Adcn_calibrated                    ' spremi varijablu Adcn_calibrated u EEPROM
Calibrated_ok_eeprom = Calibrated_ok                        ' spremi  varijablu Calibrated_ok u EEPROM
Return                                                      ' vrati se iz potprograma kalibracije

Calibration_key_free:
Debounce Calibration_key , 1 , Calibration_key_is_free      'potprogram za cekanje na otpustanje kalibracijske tipke
Goto Calibration_key_free
Calibration_key_is_free:
Return

Select_key_free:
Debounce Select_key , 1 , Select_key_is_free                'potprogram za cekanje na otpustanje Select_key tipke  uz brisanje LCD-a
Goto Select_key_free
Select_key_is_free:
Cls
Return                                                      ' obrisi LCD i vrati se iz potprograma

Next_adc:
If Channel < Channel_max Then                               ' potprogram za kruzno inkrementiranje ADC ulaza i kanala
 Incr Adcn : Incr Channel
Else
 Adcn = 0 : Channel = 1
End If
Return

Lab_exit_calibration:                                       ' kraj "Multi Adcx.inc" knjizice